package com.om.config;

import com.om.constant.LoginConstant;
import com.om.interceptor.AppRequestInterceptor;
import com.om.interceptor.JwtTokenAdminInterceptor;
import com.om.interceptor.JwtTokenUserInterceptor;
import lombok.extern.slf4j.Slf4j;

import org.springframework.context.annotation.Configuration;

import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurationSupport;

import javax.annotation.Resource;

/**
 * 配置类，注册web层相关组件
 */
@Configuration
@Slf4j
public class WebMvcConfiguration extends WebMvcConfigurationSupport {

    @Resource
    private JwtTokenAdminInterceptor jwtTokenAdminInterceptor;

    @Resource
    private JwtTokenUserInterceptor jwtTokenUserInterceptor;

    @Resource
    private AppRequestInterceptor appRequestInterceptor;
    /**
     * 注册自定义拦截器
     *
     * @param registry
     */
    protected void addInterceptors(InterceptorRegistry registry) {
        log.info("开始注册自定义拦截器...");

            registry.addInterceptor(appRequestInterceptor);
        //管理员拦截器
    /*    registry.addInterceptor(jwtTokenAdminInterceptor)
                .addPathPatterns("/web/**")
                .excludePathPatterns(LoginConstant.ADMIN_ALLOWED_PATHS)
                .excludePathPatterns(LoginConstant.KNIFE4J_PATHS);*/

      /*  //用户拦截器
        registry.addInterceptor(jwtTokenUserInterceptor)
                .addPathPatterns("/app/**")
                .excludePathPatterns(LoginConstant.USER_ALLOWED_PATHS)
                .excludePathPatterns(LoginConstant.KNIFE4J_PATHS);*/

    }




    /**
     * 设置静态资源映射
     * @param registry
     */
    protected void addResourceHandlers(ResourceHandlerRegistry registry) {
        registry.addResourceHandler("/doc.html").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("/favicon.ico").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("/webjars/**").addResourceLocations("classpath:/META-INF/resources/webjars/");
    }

    @Override
    public void addCorsMappings(CorsRegistry registry) {
        log.info("已开启CORS跨域");

        //允许所有的访问请求（访问路径）
        registry.addMapping("/**").
                //允许所有的请求方法访问该跨域资源服务器
                        allowedMethods("*").
                //允许所有的请求域名访问我们的跨域资源
                        allowedOrigins("*").
                //允许所有的请求header访问
                        allowedHeaders("*");
    }

    }


